/**
 * Any copyright is dedicated to the Public Domain.
 * http://creativecommons.org/publicdomain/zero/1.0/
 */

// Just a wrapper around SimpleTest related functions for now.
export const Assert = {
  ok,
  equal: is,
};
