.auto-stylesheet-class-1 {
  padding: 1em;
}
.auto-stylesheet-class-2 {
  padding: 2em;
}
.auto-stylesheet-class-3 {
  padding: 3em;
}

.auto-stylesheet-class-1,
div.auto-stylesheet-class-2,
p:first-of-type.auto-stylesheet-class-3,
.auto-stylesheet-class-4 {
  background-color: blue;
}

:root .auto-bold .auto-stylesheet-class-5 {
  font-size: bold;
}
